@@ -28,7 +28,6 @@
 #include "electron.h"
 #include "text.h"
 #include "plugin.h"
-#include "mendeleiev.h"
 #include "mesomer.h"
 #include "mesomery.h"
 #include "mesomery-arrow.h"
@@ -51,12 +50,10 @@
 #include <openbabel/mol.h>
 #include <openbabel/obconversion.h>
 
-bool new_ui;
-
 // Objects creation static methods
-static Object* CreateMolecule()
+static Object* CreateMolecule ()
 {
-	return new gcpMolecule();
+	return new gcpMolecule ();
 }
 
 static Object* CreateReaction ()
@@ -69,12 +66,12 @@ static Object* CreateReactant ()
 	return new gcpReactant ();
 }
 
-static Object* CreateReactionStep()
+static Object* CreateReactionStep ()
 {
 	return new gcpReactionStep ();
 }
 
-static Object* CreateReactionArrow()
+static Object* CreateReactionArrow ()
 {
 	return new gcpReactionArrow (NULL);
 }
@@ -84,9 +81,9 @@ static Object* CreateMesomery ()
 	return new gcpMesomery ();
 }
 
-static Object* CreateMesomeryArrow()
+static Object* CreateMesomeryArrow ()
 {
-	return new gcpMesomeryArrow(NULL);
+	return new gcpMesomeryArrow (NULL);
 }
 
 static Object* CreateMesomer ()
@@ -94,26 +91,33 @@ static Object* CreateMesomer ()
 	return new gcpMesomer ();
 }
 
-static Object* CreateText()
+static Object* CreateText ()
 {
-	return new gcpText();
+	return new gcpText ();
 }
 
-static Object* CreateFragment()
+static Object* CreateFragment ()
 {
-	return new gcpFragment();
+	return new gcpFragment ();
 }
 
 list<gcpApplication*> Apps;
 bool	gcpApplication::m_bInit = false;
 bool	gcpApplication::m_Have_Ghemical = false;
 bool	gcpApplication::m_Have_InChI = false;
-map<string, string> gcpApplication::m_SupportedPixbufFormats;
 
 #warning "the following lines should be removed for stable releases"
 #undef PACKAGE
 #define PACKAGE "gchempaint-unstable" 
 
+static char *mime_types[] = {
+	"application/x-gchempaint",
+	"chemical/x-cml",
+	"chemical/x-mdl-molfile",
+	"chemical/x-pdb",
+	"chemical/x-chemdraw",
+};
+
 gcpApplication::gcpApplication (): Application ("GChemPaint", DATADIR, PACKAGE, "gchempaint")
 {
 	Apps.push_back(this);
@@ -122,7 +126,7 @@ gcpApplication::gcpApplication (): Application ("GChemPaint", DATADIR, PACKAGE,
 	m_pActiveTool = NULL;
 	m_pNode = NULL;
 	m_NumWindow = 1;
-	FontName = g_strdup("Bitstream Vera Serif 12");
+//	FontName = g_strdup ("Bitstream Vera Serif 12");
 
 	if (!m_bInit) {
 		// Check for programs
@@ -148,25 +152,7 @@ gcpApplication::gcpApplication (): Application ("GChemPaint", DATADIR, PACKAGE,
 			g_free (errors);
 			errors = NULL;
 		}
-		// check supported pixbuf formats
-		GSList *formats = gdk_pixbuf_get_formats ();
-		GSList *l = formats;
-		GdkPixbufFormat *format;
-		char **mimes, **mime;
-		while (l) {
-			format = (GdkPixbufFormat*) l->data;
-			if (gdk_pixbuf_format_is_writable (format)) {
-				mimes = gdk_pixbuf_format_get_mime_types (format);
-				mime = mimes;
-				while (*mime)
-					m_SupportedPixbufFormats[*mime++] = gdk_pixbuf_format_get_name (format);
-				g_strfreev (mimes);
-			}
-			l = l->next;
-		}
-		g_slist_free (formats);
 
-		gcpText::InitTags(FontName);
 		// Initialize types
 		Object::AddType ("molecule", CreateMolecule, MoleculeType);
 		Object::AddType ("reaction", CreateReaction, ReactionType);
@@ -211,7 +197,6 @@ gcpApplication::gcpApplication (): Application ("GChemPaint", DATADIR, PACKAGE,
 	g_object_unref (G_OBJECT (IconFactory));
 	XmlDoc = xmlNewDoc ((xmlChar const*) "1.0");
 	visible_windows = 0;
-	m_idle = 0;
 	m_SupportedMimeTypes.push_back ("application/x-gchempaint");
 	m_SupportedMimeTypes.push_back ("chemical/x-chemdraw");
 	m_SupportedMimeTypes.push_back ("chemical/x-cml");
@@ -226,7 +211,6 @@ gcpApplication::~gcpApplication()
 		delete (*tool).second;
 	Tools.clear ();
 	Apps.remove (this);
-	g_free (FontName);
 	if (m_pNode)
 		bonobo_ui_node_free (m_pNode);
 	if (XmlDoc)
@@ -234,14 +218,6 @@ gcpApplication::~gcpApplication()
 	m_SupportedMimeTypes.clear ();
 }
 
-void gcpApplication::ActivateMenu(const string& menuname, bool activate)
-{
-}
-
-void gcpApplication::ActivateToolItem(const string& itemname, bool activate)
-{
-}
-
 void gcpApplication::ActivateTool(const string& toolname, bool activate)
 {
 	if (Tools[toolname]) {
@@ -265,10 +241,20 @@ void gcpApplication::ActivateTool(const string& toolname, bool activate)
 
 void gcpApplication::ClearStatus()
 {
+	if (m_pActiveDoc) {
+		gcpWindow *Win = m_pActiveDoc->GetWindow ();
+		if (Win)
+			Win->ClearStatus ();
+	}
 }
 
 void gcpApplication::SetStatusText(const char* text)
 {
+	if (m_pActiveDoc) {
+		gcpWindow *Win = m_pActiveDoc->GetWindow ();
+		if (Win)
+			Win->SetStatusText (text);
+	}
 }
 
 void gcpApplication::OnSaveAs()
@@ -318,7 +304,14 @@ void gcpApplication::AddUI (const char* filename)
 	}
 }
 
-bool gcpApplication::FileProcess(const gchar* filename, const gchar* mime_type, bool bSave, GtkWindow *window, Document *Doc)
+enum {
+	CHEMISTRY,
+	SVG,
+	EPS,
+	PIXBUF
+};
+
+bool gcpApplication::FileProcess (const gchar* filename, const gchar* mime_type, bool bSave, GtkWindow *window, Document *Doc)
 {
 	const gchar* ext;
 	gcpDocument *pDoc = static_cast<gcpDocument*> (Doc);
@@ -332,9 +325,12 @@ bool gcpApplication::FileProcess(const gchar* filename, const gchar* mime_type,
 		return true;
 	}
 	if (!mime_type)
-		mime_type = "application/x-gchempaint";
+		mime_type = "";
 	int filetype = (mime_type)? 0: gcp_ft_Normal;
+	int image_type = CHEMISTRY;
 	int n = strlen (filename), i = n - 1;
+	char const *pixbuf_type = NULL;
+	string filename2 = filename;
 	while ((i > 0) && (filename[i] != '.') && (filename[i] != '/')) i--;
 	if (filename[i] == '/') i = 0;
 	ext = (i > 0)? filename + i: NULL;
@@ -350,179 +346,210 @@ bool gcpApplication::FileProcess(const gchar* filename, const gchar* mime_type,
 			filetype = gcp_ft_PDB;
 		else if (!strcmp (mime_type, "chemical/x-chemdraw"))
 			filetype = gcp_ft_ChemDraw;
-		else {
+		else if (!strcmp (mime_type, "image/svg+xml"))
+			image_type = SVG;
+		else if (!strcmp (mime_type, "image/x-eps"))
+			image_type = EPS;
+		else if ((pixbuf_type = GetPixbufTypeName (filename2, mime_type)))
+			image_type = PIXBUF;
+		if (!filetype && (!bSave || !image_type)) {
 			GtkWidget* message = gtk_message_dialog_new (window, GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, 
 														_("Sorry, format not supported!"));
-			gtk_window_set_icon_name (GTK_WINDOW (message), "gchempaint");
 			gtk_dialog_run (GTK_DIALOG (message));
 			gtk_widget_destroy (message);
 			return true;
 		}
 	}
 	if (!ext)
-		switch (filetype)
-		{
-			case gcp_ft_Normal: ext = ".gchempaint"; break;
-			case gcp_ft_CML: ext = ".cml"; break;
-			case gcp_ft_SDF:
-				if (!strcmp(filename + n - 3, "sdf")) ext = ".sdf";
-				else if (!strcmp(filename + n - 2, "sd")) ext = ".sd";
-				else if (bSave) ext = ".mol";
+		if (image_type)
+			switch (image_type) {
+			case SVG:
+				ext = ".svg";
 				break;
-			case gcp_ft_PDB:
-				if (!strcmp(filename + n - 3, "ent")) ext = ".ent";
-				else if (bSave) ext = ".pdb";
-				break;
-			case gcp_ft_ChemDraw: ext = ".chm"; break;
+			case EPS:
+				ext = ".eps";
+				break; 
 			default:
 				ext = NULL;
 				break;
-		}
-	gchar* filename2;
+			}
+		else
+			switch (filetype) {
+			case gcp_ft_Normal:
+				ext = ".gchempaint";
+				break;
+			case gcp_ft_CML:
+				ext = ".cml";
+				break;
+			case gcp_ft_SDF:
+				if (!strcmp (filename + n - 4, ".sdf")) 
+					ext = ".sdf";
+				else if (!strcmp (filename + n - 3, ".sd"))
+					ext = ".sd";
+				else if (bSave)
+					ext = ".mol";
+				break;
+			case gcp_ft_PDB:
+				if (!strcmp(filename + n - 4, ".ent"))
+					ext = ".ent";
+				else if (bSave)
+					ext = ".pdb";
+				break;
+			case gcp_ft_ChemDraw:
+				ext = ".chm";
+				break;
+			default:
+				ext = NULL;
+				break;
+			}
 	bool err;
 	GnomeVFSURI *uri;
-	if (bSave)
-	{
+	if (bSave) {
 		if (ext)
 		{
-			int i = strlen(filename) - strlen(ext);
-			if ((i > 0) && (!strcmp(filename +i, ext)))
-				filename2 = g_strdup(filename);
-			else
-				filename2 = g_strdup_printf("%s%s", filename, ext);
+			int i = strlen (filename) - strlen(ext);
+			if ((i <= 0) || (strcmp (filename + i, ext)))
+				filename2 += ext;
 		}
-		else filename2 = g_strdup(filename);
-		uri = gnome_vfs_uri_new (filename2);
+		uri = gnome_vfs_uri_new (filename2.c_str ());
 		err = gnome_vfs_uri_exists (uri);
 		gnome_vfs_uri_unref (uri);
 		gint result = GTK_RESPONSE_YES;
-		if (err)
-		{
-			gchar * message = g_strdup_printf(_("File %s\nexists, overwrite?"), filename2);
-			GtkDialog* Box = GTK_DIALOG(gtk_message_dialog_new(NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_QUESTION, GTK_BUTTONS_YES_NO, message));
+		if (err) {
+			gchar * message = g_strdup_printf (_("File %s\nexists, overwrite?"), filename2.c_str ());
+			GtkDialog* Box = GTK_DIALOG (gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_QUESTION, GTK_BUTTONS_YES_NO, message));
 			gtk_window_set_icon_name (GTK_WINDOW (Box), "gchempaint");
-			result = gtk_dialog_run(Box);
-			gtk_widget_destroy(GTK_WIDGET(Box));
-			g_free(message);
+			result = gtk_dialog_run (Box);
+			gtk_widget_destroy (GTK_WIDGET (Box));
+			g_free (message);
 		}
-		if (result == GTK_RESPONSE_YES)
-		{
+		if (result == GTK_RESPONSE_YES) {
 			// destroy the old file
-			gnome_vfs_unlink (filename2);
+			gnome_vfs_unlink (filename2.c_str ());
 			pDoc->SetType (filetype);
-			if (filetype == gcp_ft_Normal)
-				SaveGcp (filename2, pDoc);
-			else
-				SaveWithBabel (filename2, ext, pDoc, filetype);
+			switch (image_type) {
+			case SVG:
+				m_pActiveDoc->ExportImage (filename2, "svg");
+				break;
+			case EPS:
+				m_pActiveDoc->ExportImage (filename2, "eps");
+				break;
+			case PIXBUF:
+				m_pActiveDoc->ExportImage (filename2, pixbuf_type, GetImageResolution ());
+				break;
+			default:
+				if (filetype == gcp_ft_Normal)
+					SaveGcp (filename2, pDoc);
+				else
+					SaveWithBabel (filename2, ext, pDoc, filetype);
+			}
 		}
-	}
-	else //loading
-	{
+	} else  { //loading
 		uri = gnome_vfs_uri_new (filename);
 		err = !gnome_vfs_uri_exists (uri);
 		gnome_vfs_uri_unref (uri);
-		if (err)
-		{
+		if (err) {
 			if (!ext)
-				switch (filetype)
-				{
-					case gcp_ft_SDF:
-						filename2 = g_strdup_printf("%s.%s",filename, "mol");
-						uri = gnome_vfs_uri_new (filename2);
-						err = !gnome_vfs_uri_exists (uri);
-						gnome_vfs_uri_unref (uri);
-						if (!err)
-							break;
-						g_free(filename2);
-						filename2 = g_strdup_printf("%s.%s",filename, "sdf");
-						uri = gnome_vfs_uri_new (filename2);
-						err = !gnome_vfs_uri_exists (uri);
-						gnome_vfs_uri_unref (uri);
-						if (!err)
-							break;
-						g_free(filename2);
-						filename2 = g_strdup_printf("%s.%s",filename, "sd");
-						uri = gnome_vfs_uri_new (filename2);
-						err = !gnome_vfs_uri_exists (uri);
-						gnome_vfs_uri_unref (uri);
-						if (!err)
-							break;
-						g_free(filename2);
-						filename2 = g_strdup(filename);
+				switch (filetype) {
+				case gcp_ft_SDF:
+					filename2 = string (filename) + ".mol";
+					uri = gnome_vfs_uri_new (filename2.c_str ());
+					err = !gnome_vfs_uri_exists (uri);
+					gnome_vfs_uri_unref (uri);
+					if (!err)
 						break;
-					case gcp_ft_PDB:
-						filename2 = g_strdup_printf("%s.%s",filename, "pdb");
-						uri = gnome_vfs_uri_new (filename2);
-						err = !gnome_vfs_uri_exists (uri);
-						gnome_vfs_uri_unref (uri);
-						if (!err)
-							break;
-						g_free(filename2);
-						filename2 = g_strdup_printf("%s.%s",filename, "ent");
-						uri = gnome_vfs_uri_new (filename2);
-						err = !gnome_vfs_uri_exists (uri);
-						gnome_vfs_uri_unref (uri);
-						if (!err)
-							break;
-						g_free(filename2);
-						filename2 = g_strdup(filename);
+					filename2 = string (filename) + ".sdf";
+					uri = gnome_vfs_uri_new (filename2.c_str ());
+					err = !gnome_vfs_uri_exists (uri);
+					gnome_vfs_uri_unref (uri);
+					if (!err)
 						break;
-					default:
-						filename2 = g_strdup(filename);
+					filename2 = string (filename) + ".sd";
+					uri = gnome_vfs_uri_new (filename2.c_str ());
+					err = !gnome_vfs_uri_exists (uri);
+					gnome_vfs_uri_unref (uri);
+					if (!err)
+						break;
+					filename2 = filename;
+					break;
+				case gcp_ft_PDB:
+					filename2 = string (filename) + ".pdb";
+					uri = gnome_vfs_uri_new (filename2.c_str ());
+					err = !gnome_vfs_uri_exists (uri);
+					gnome_vfs_uri_unref (uri);
+					if (!err)
+						break;
+					filename2 = string (filename) + ".ent";
+					uri = gnome_vfs_uri_new (filename2.c_str ());
+					err = !gnome_vfs_uri_exists (uri);
+					gnome_vfs_uri_unref (uri);
+					if (!err)
+						break;
+					filename2 = filename;
+					break;
 				}
-			else filename2 = g_strdup_printf("%s%s",filename, ext);
-			uri = gnome_vfs_uri_new (filename2);
+			else
+				filename2 += ext;
+			uri = gnome_vfs_uri_new (filename2.c_str ());
 			err = !gnome_vfs_uri_exists (uri);
 			gnome_vfs_uri_unref (uri);
-			if (err)
-			{
-				g_free(filename2);
-				filename2 = g_strdup(filename);
+			if (err) {
+				filename2 = filename;
 			}
 		}
-		else filename2 = g_strdup(filename);
 		if (filetype == gcp_ft_Normal)
 			OpenGcp (filename2, pDoc);
 		else
 			OpenWithBabel (filename2, ext, pDoc, filetype);
 	}
-	g_free(filename2);
 	return false;
 }
 
-void gcpApplication::SaveWithBabel(const gchar* filename, const gchar* ext, gcpDocument* pDoc, int Type)
+void gcpApplication::SaveWithBabel (string const &filename, const gchar* ext, gcpDocument* pDoc, int Type)
 {
-	pDoc->SetFileName(filename, ext);
-	pDoc->SetType(Type);
-	pDoc->Save();
+	pDoc->SetFileName (filename, ext);
+	pDoc->SetType (Type);
+	pDoc->Save ();
+	GtkRecentData data;
+	data.display_name = (char*) pDoc->GetTitle ();
+	data.description = NULL;
+	data.mime_type = mime_types[Type - 1];
+	data.app_name = "gchempaint";
+	data.app_exec = "gchempaint %u";
+	data.groups = NULL;
+	data.is_private =  FALSE;
+	gtk_recent_manager_add_full (GetRecentManager (), filename.c_str (), &data);
 }
 
-void gcpApplication::OpenWithBabel(const gchar* filename, const gchar* ext, gcpDocument* pDoc, int Type)
+void gcpApplication::OpenWithBabel (string const &filename, const gchar* ext, gcpDocument* pDoc, int Type)
 {
 	char *old_num_locale;
 	bool bNew = (pDoc == NULL), local;
 	GnomeVFSFileInfo *info = NULL;
 	bool result;
-	try
-	{
-		if (!filename) throw (int) 0;
+	try {
+		if (!filename.length ())
+			throw (int) 0;
 		info = gnome_vfs_file_info_new ();
-		gnome_vfs_get_file_info (filename, info, GNOME_VFS_FILE_INFO_DEFAULT);
+		gnome_vfs_get_file_info (filename.c_str (), info, GNOME_VFS_FILE_INFO_DEFAULT);
 		local = GNOME_VFS_FILE_INFO_LOCAL (info);
 		gnome_vfs_file_info_unref (info);
-		if (bNew)
-			pDoc = new gcpDocument (this, true);
+		if (bNew) {
+			OnFileNew ();
+			pDoc = m_pActiveDoc;
+		}
 		if (local) {
 			ifstream ifs;
-			GnomeVFSURI *uri = gnome_vfs_uri_new (filename);
+			GnomeVFSURI *uri = gnome_vfs_uri_new (filename.c_str ());
 			ifs.open (gnome_vfs_uri_get_path (uri));
 			gnome_vfs_uri_unref (uri);
-			if (!ifs) throw (int) 1;
+			if (ifs.fail ())
+				throw (int) 1;
 			old_num_locale = g_strdup (setlocale (LC_NUMERIC, NULL));
 			setlocale(LC_NUMERIC, "C");
 			OBMol Mol;
 			OBConversion Conv;
-			OBFormat* pInFormat = Conv.FormatFromExt (filename);
+			OBFormat* pInFormat = Conv.FormatFromExt (filename.c_str ());
 			Conv.SetInAndOutFormats (pInFormat, pInFormat);
 			Conv.Read (&Mol, &ifs);
 			result = pDoc->ImportOB(Mol);
@@ -532,14 +559,14 @@ void gcpApplication::OpenWithBabel(const gchar* filename, const gchar* ext, gcpD
 		} else {
 			char *buf;
 			int size;
-			if (gnome_vfs_read_entire_file (filename, &size, &buf) != GNOME_VFS_OK)
+			if (gnome_vfs_read_entire_file (filename.c_str (), &size, &buf) != GNOME_VFS_OK)
 				throw 1;
 			istringstream iss (buf);
 			old_num_locale = g_strdup (setlocale (LC_NUMERIC, NULL));
 			setlocale(LC_NUMERIC, "C");
 			OBMol Mol;
 			OBConversion Conv;
-			OBFormat* pInFormat = Conv.FormatFromExt (filename);
+			OBFormat* pInFormat = Conv.FormatFromExt (filename.c_str ());
 			Conv.SetInAndOutFormats (pInFormat, pInFormat);
 			Conv.Read (&Mol, &iss);
 			result = pDoc->ImportOB(Mol);
@@ -553,9 +580,19 @@ void gcpApplication::OpenWithBabel(const gchar* filename, const gchar* ext, gcpD
 			if (bNew) delete pDoc;
 			throw (int) 2;
 		}
-		pDoc->SetFileName(filename, ext);
-		pDoc->SetType(Type);
-		if (bNew) AddDocument(pDoc);
+		pDoc->SetFileName (filename, ext);
+		pDoc->SetType (Type);
+		if (bNew)
+			AddDocument (pDoc);
+		GtkRecentData data;
+		data.display_name = (char*) pDoc->GetTitle ();
+		data.description = NULL;
+		data.mime_type = mime_types[Type - 1];
+		data.app_name = "gchempaint";
+		data.app_exec = "gchempaint %u";
+		data.groups = NULL;
+		data.is_private =  FALSE;
+		gtk_recent_manager_add_full (GetRecentManager (), filename.c_str (), &data);
 	}
 	catch (int num)
 	{
@@ -567,10 +604,10 @@ void gcpApplication::OpenWithBabel(const gchar* filename, const gchar* ext, gcpD
 			mess = g_strdup_printf(_("No filename"));
 			break;
 		case 1:
-			mess = g_strdup_printf(_("Could not open file\n%s"), filename);
+			mess = g_strdup_printf(_("Could not open file\n%s"), filename.c_str ());
 			break;
 		case 2:
-			mess = g_strdup_printf(_("%s: parse error."), filename);
+			mess = g_strdup_printf(_("%s: parse error."), filename.c_str ());
 			break;
 		default:
 			throw (num); //this should not occur
@@ -583,10 +620,19 @@ void gcpApplication::OpenWithBabel(const gchar* filename, const gchar* ext, gcpD
 	}
 }
 
-void gcpApplication::SaveGcp(const gchar* filename, gcpDocument* pDoc)
+void gcpApplication::SaveGcp (string const &filename, gcpDocument* pDoc)
 {
-	pDoc->SetFileName(filename, ".gchempaint");
-	pDoc->Save();
+	pDoc->SetFileName (filename, ".gchempaint");
+	pDoc->Save ();
+	GtkRecentData data;
+	data.display_name = (char*) pDoc->GetTitle ();
+	data.description = NULL;
+	data.mime_type = "application/x-gchempaint";
+	data.app_name = "gchempaint";
+	data.app_exec = "gchempaint %u";
+	data.groups = NULL;
+	data.is_private =  FALSE;
+	gtk_recent_manager_add_full (GetRecentManager (), filename.c_str (), &data);
 }
 
 static int	cb_vfs_to_xml (GnomeVFSHandle *handle, char* buf, int nb)
@@ -595,31 +641,32 @@ static int	cb_vfs_to_xml (GnomeVFSHandle *handle, char* buf, int nb)
 	return  (gnome_vfs_read (handle, buf, nb, &ndone) == GNOME_VFS_OK)? (int) ndone: -1;
 }
 
-void gcpApplication::OpenGcp(const gchar* filename, gcpDocument* pDoc)
+void gcpApplication::OpenGcp (string const &filename, gcpDocument* pDoc)
 {
 	xmlDocPtr xml = NULL;
 	char *old_num_locale, *old_time_locale;
 	GnomeVFSFileInfo *info = NULL;
+	bool create = false;
 	try
 	{
-		if (!filename)
+		if (!filename.length ())
 			throw (int) 0;
 
 		// try opening with write access to see if it is readonly
 		// use xmlReadIO for non local files.
 		info = gnome_vfs_file_info_new ();
-		gnome_vfs_get_file_info (filename, info, GNOME_VFS_FILE_INFO_DEFAULT);
+		gnome_vfs_get_file_info (filename.c_str (), info, GNOME_VFS_FILE_INFO_DEFAULT);
 
 		if (GNOME_VFS_FILE_INFO_LOCAL (info)) {
-			if (!(xml = xmlParseFile(filename)))
+			if (!(xml = xmlParseFile(filename.c_str ())))
 				throw (int) 1;
 		} else {
 			GnomeVFSHandle *handle;
-			GnomeVFSResult result = gnome_vfs_open (&handle, filename, GNOME_VFS_OPEN_READ);
+			GnomeVFSResult result = gnome_vfs_open (&handle, filename.c_str (), GNOME_VFS_OPEN_READ);
 			if (result != GNOME_VFS_OK)
 				throw 1;
 			if (!(xml = xmlReadIO ((xmlInputReadCallback) cb_vfs_to_xml, 
-					 (xmlInputCloseCallback) gnome_vfs_close, handle, filename, NULL, 0)))
+					 (xmlInputCloseCallback) gnome_vfs_close, handle, filename.c_str (), NULL, 0)))
 				throw 1;
 		}
 		if (xml->children == NULL) throw (int) 2;
@@ -629,6 +676,7 @@ void gcpApplication::OpenGcp(const gchar* filename, gcpDocument* pDoc)
 		old_time_locale = g_strdup(setlocale(LC_TIME, NULL));
 		setlocale(LC_TIME, "C");
 		if (!pDoc) {
+			create = true;
 			OnFileNew ();
 			pDoc = m_pActiveDoc;
 		}
@@ -638,15 +686,23 @@ void gcpApplication::OpenGcp(const gchar* filename, gcpDocument* pDoc)
 		g_free(old_num_locale);
 		setlocale(LC_TIME, old_time_locale);
 		g_free(old_time_locale);
-		if (!result)
-		{
-			OnFileClose ();
+		if (!result) {
+			if (create)
+				delete pDoc;
 			throw (int) 4;
 		}
 		if (!(info->permissions & (GNOME_VFS_PERM_USER_WRITE | GNOME_VFS_PERM_GROUP_WRITE)))
 			pDoc->SetReadOnly (true);
 		gnome_vfs_file_info_unref (info);
 		xmlFreeDoc(xml);
+		GtkRecentData data;
+		data.display_name = (char*) pDoc->GetTitle ();
+		data.description = NULL;
+		data.mime_type = "application/x-gchempaint";
+		data.app_name = "gchempaint";
+		data.app_exec = "gchempaint %u";
+		data.groups = NULL;
+		gtk_recent_manager_add_full (GetRecentManager (), filename.c_str (), &data);
 	}
 	catch (int num)
 	{
@@ -661,16 +717,16 @@ void gcpApplication::OpenGcp(const gchar* filename, gcpDocument* pDoc)
 			mess = g_strdup_printf(_("No filename"));
 			break;
 		case 1:
-			mess = g_strdup_printf(_("Could not load file\n%s"), filename);
+			mess = g_strdup_printf(_("Could not load file\n%s"), filename.c_str ());
 			break;
 		case 2:
-			mess = g_strdup_printf(_("%s: invalid xml file.\nTree is empty?"), filename);
+			mess = g_strdup_printf(_("%s: invalid xml file.\nTree is empty?"), filename.c_str ());
 			break;
 		case 3:
-			mess = g_strdup_printf(_("%s: invalid file format."), filename);
+			mess = g_strdup_printf(_("%s: invalid file format."), filename.c_str ());
 			break;
 		case 4:
-			mess = g_strdup_printf(_("%s: parse error."), filename);
+			mess = g_strdup_printf(_("%s: parse error."), filename.c_str ());
 			break;
 		default:
 			throw (num); //this should not occur
@@ -695,12 +751,6 @@ void gcpApplication::MergeNodes(BonoboUINode* Dest, BonoboUINode* Source)
 	}
 }
 
-void gcpApplication::SetFontName(char* fontname)
-{
-	g_free (FontName);
-	FontName = g_strdup (fontname);
-}
-
 void gcpApplication::InitTools()
 {
 	map<string, gcpTool*>::iterator i = Tools.begin (), end = Tools.end ();
@@ -709,80 +759,17 @@ void gcpApplication::InitTools()
 			(*i).second->Activate ((*i).first == "Select");
 }
 
-void gcpApplication::OnShowElements ()
-{
-	gcpMendeleievDlg* dlg = (gcpMendeleievDlg*) GetDialog("Mendeleiev");
-	if (!dlg)
-	{
-		dlg = new gcpMendeleievDlg (this, GetCurZ());
-		ToggleMenu ("Mendeleiev", true);
-	}
-	else gdk_window_raise (GTK_WIDGET(dlg->GetWindow())->window); 
-}
-
 void gcpApplication::OnSaveAsImage ()
 {
-	char* filename;
-	GtkFileChooser *dialog = (GtkFileChooser*) gtk_file_chooser_dialog_new (
-															_("Save as image"),
-															GetWindow(),
-															GTK_FILE_CHOOSER_ACTION_SAVE,
-															GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
-															GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-															NULL);
-	gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
-	GtkFileFilter* filter = gtk_file_filter_new ();
-	map<string, string>::iterator i, end = m_SupportedPixbufFormats.end ();
+	if (!m_pActiveDoc)
+		return;
+	list<char const*> l;
+	map<string, GdkPixbufFormat*>::iterator i, end = m_SupportedPixbufFormats.end ();
 	for (i = m_SupportedPixbufFormats.begin (); i != end; i++)
-		gtk_file_filter_add_mime_type (filter, (*i).first.c_str ());
-	gtk_file_filter_add_mime_type (filter, "image/svg+xml");
-	gtk_file_filter_add_mime_type (filter, "image/x-eps");
-	gtk_file_filter_add_mime_type (filter, "image/x-xfig");
-	gtk_file_chooser_set_filter (dialog, filter);
-	while (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT) {
-		filename = gtk_file_chooser_get_filename (dialog);
-		if (!filename || !strlen(filename) || filename[strlen(filename) - 1] == '/') {
-			GtkWidget* message = gtk_message_dialog_new (GTK_WINDOW (dialog), GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, 
-																_("Please enter a file name,\nnot a directory"));
-			gtk_window_set_icon_name (GTK_WINDOW (message), "gchempaint");
-			gtk_dialog_run (GTK_DIALOG (message));
-			gtk_widget_destroy (message);
-			continue;
-		}
-		struct stat buf;
-		gint err;
-		err = stat(filename, &buf);
-		gint result = GTK_RESPONSE_YES;
-		if (!err)
-		{
-			gchar * message = g_strdup_printf(_("File %s\nexists, overwrite?"), filename);
-			GtkDialog* Box = GTK_DIALOG(gtk_message_dialog_new(NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_QUESTION, GTK_BUTTONS_YES_NO, message));
-			gtk_window_set_icon_name (GTK_WINDOW (Box), "gchempaint");
-			result = gtk_dialog_run(Box);
-			gtk_widget_destroy(GTK_WIDGET(Box));
-			g_free(message);
-			if (result != GTK_RESPONSE_YES)
-				continue;
-		}
-		const char* mime_type = gnome_vfs_mime_type_from_name (filename);
-		if (m_SupportedPixbufFormats.find (mime_type) != m_SupportedPixbufFormats.end())
-			m_pActiveDoc->ExportImage (filename, m_SupportedPixbufFormats[mime_type].c_str ());
-		else if (!strcmp (mime_type, "image/x-eps"))
-				m_pActiveDoc->ExportImage (filename, "eps");
-		else if (!strcmp (mime_type, "image/svg+xml")) {
-			m_pActiveDoc->ExportImage (filename, "svg");
-		} else {
-			GtkWidget* message = gtk_message_dialog_new (GTK_WINDOW (dialog), GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, 
-																_("Sorry, format not supported!"));
-			gtk_window_set_icon_name (GTK_WINDOW (message), "gchempaint");
-			gtk_dialog_run (GTK_DIALOG (message));
-			gtk_widget_destroy (message);
-			continue;
-		}
-		break;//replace with sensible code
-	}
-	gtk_widget_destroy (GTK_WIDGET (dialog));
-	return;	
+		l.push_front ((*i).first.c_str ());
+	l.push_front ("image/x-eps");
+	l.push_front ("image/svg+xml");
+	FileChooser (this, true, l, m_pActiveDoc, _("Save as image"), GetImageResolutionWidget ());
 }
 
 void gcpApplication::Zoom (double zoom)
@@ -845,8 +832,8 @@ void gcpApplication::BuildTools ()
 	m_pActiveTool = Tools["Select"];
 	if (m_pActiveTool)
 		m_pActiveTool->Activate(true);
-	ToolsBox->OnSelectTool (m_pActiveTool);		
-	gcpMendeleievDlg::SetElement (this, m_CurZ);
+	ToolsBox->OnSelectTool (m_pActiveTool);	
+	ToolsBox->OnElementChanged (m_CurZ);	
 }
 
 void gcpApplication::ShowTools (bool visible)
@@ -922,28 +909,15 @@ void gcpApplication::NotifyIconification (bool iconified)
 	}
 }
 
-static bool on_wait_focus (gcpApplication *App)
-{
-	App->CheckFocus ();
-	return false;
-}
-
-void gcpApplication::CheckFocus ()
-{
-	ShowTools (focused || gtk_grab_get_current ());
-	m_idle = 0;
-}
-
 void gcpApplication::NotifyFocus (bool has_focus, gcpWindow *window)
 {
 	if (window) {
 		m_pActiveWin = window;
 		m_pActiveDoc = window->GetDocument ();
 		m_pActiveTool->Activate ();
+		if (has_focus)
+			ShowTools (true);
 	}
-	focused = has_focus;
-	if (!m_idle)
-		g_idle_add ((GSourceFunc) on_wait_focus, this);
 }
 
 void gcpApplication::CloseAll ()
@@ -951,3 +925,10 @@ void gcpApplication::CloseAll ()
 	while (!m_Windows.empty ())
 		(*m_Windows.begin ())->Destroy ();
 }
+
+void gcpApplication::ActivateWindowsActionWidget (const char *path, bool activate)
+{
+	std::set<gcpWindow*>::iterator i, iend = m_Windows.end ();
+	for (i = m_Windows.begin (); i != iend; i++)
+		(*i)->ActivateActionWidget (path, activate);
+}
