@@ -4,9 +4,7 @@
  * GChemPaint library
  * document.cc 
  *
- * Copyright (C) 2001-2005
- *
- * Developed by Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2001-2006 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -24,6 +22,8 @@
  * USA
  */
 
+#undef G_DISABLE_DEPRECATED
+
 #include "gchempaint-config.h"
 #include "application.h"
 #include "view.h"
@@ -33,6 +33,8 @@
 #include "reaction.h"
 #include "mesomery.h"
 #include "text.h"
+#include "theme.h"
+#include "window.h"
 #include <locale.h>
 #include <unistd.h>
 #include <libgen.h>
@@ -48,30 +50,46 @@ using namespace std;
 SignalId OnChangedSignal;
 SignalId OnDeleteSignal;
 
-gcpDocument::gcpDocument(gcpApplication *App, bool StandAlone): Document()
+gcpDocument::gcpDocument(gcpApplication *App, bool StandAlone, gcpWindow *window):
+	Document (App),
+	m_OpID (0),
+	m_LastStackSize (0)
 {
 	m_pApp = App;
 	m_pView = NULL;
+	m_Window = window;
 	m_filename = NULL;
 	m_label = NULL;
 	m_title = NULL;
+	m_bWriteable = true;
+	m_PangoAttrList = pango_attr_list_new ();
+	SetTheme (ThemeManager.GetTheme ("Default"));
 	m_pView = new gcpView(this, !StandAlone);
 	m_bDirty = m_bIsLoading = m_bUndoRedo = false;
-	m_bWriteable = true;
 	m_FileType = gcp_ft_Normal;
 	m_DocPropDlg = NULL;
 	g_date_set_time(&CreationDate, time(NULL));
 	g_date_clear(&RevisionDate, 1);
-	const char* chn = getenv("USERNAME");
-	if (chn) m_author = g_strdup(chn);
-	else m_author = NULL;
-	chn = getenv("E_MAIL");
-	if (chn) m_mail = g_strdup(chn);
-	else m_mail = NULL;
+	const char* chn = getenv ("REAL_NAME");
+	if (!chn)
+		chn = getenv ("USERNAME");
+	if (chn)
+		m_author = g_strdup(chn);
+	else
+		m_author = NULL;
+	chn = getenv ("E_MAIL");
+	if (!chn)
+		chn = getenv ("EMAIL_ADDRESS");
+	if (chn)
+		m_mail = g_strdup (chn);
+	else
+		m_mail = NULL;
 	m_comment = NULL;
 	m_pCurOp = NULL;
 	m_bReadOnly = false;
-	SetActive();
+	SetActive ();
+	if (window && App)
+		App->AddWindow (window);
 }
 
 gcpDocument::~gcpDocument ()
@@ -101,9 +119,11 @@ gcpDocument::~gcpDocument ()
 		obj->Lock ();
 		Remove (obj);
 	}
-	m_pApp->RemoveDocument (this);
+	if (m_pApp)
+		m_pApp->RemoveDocument (this);
 	if (m_pView)
 		delete m_pView;
+	pango_attr_list_unref (m_PangoAttrList);
 }
 
 GtkWidget* gcpDocument::GetWidget()
@@ -123,13 +143,14 @@ const gchar* gcpDocument::GetLabel()
 	return m_label;
 }
 
-void gcpDocument::SetFileName (const gchar* Name, const gchar* ext)
+void gcpDocument::SetFileName (string const &Name, const gchar* ext)
 {
-	if (m_filename) g_free (m_filename);
-	m_filename = g_strdup (Name);
-	m_pApp->SetCurDir (dirname (m_filename));
-	g_free (m_filename);
-	m_filename = g_strdup (Name);
+	if (m_filename)
+		g_free (m_filename);
+	m_filename = g_strdup (Name.c_str ());
+	char *path = g_path_get_dirname (m_filename);
+	m_pApp->SetCurDir (path);
+	g_free (path);
 	int i = strlen (m_filename) - 1;
 	while ((m_filename[i] != '/') && (i >= 0))
 		i--;
@@ -137,12 +158,14 @@ void gcpDocument::SetFileName (const gchar* Name, const gchar* ext)
 	int j = strlen (m_filename) - 1;
 	while ((i < j) && (m_filename[j] != '.'))
 		j--;
-	if (m_label) g_free(m_label);
-	if (strcmp(m_filename + j, ext))
+	if (m_label)
+		g_free (m_label);
+	if (strcmp (m_filename + j, ext))
 		m_label = g_strdup (m_filename + i);
 	else
 		m_label = g_strndup (m_filename + i, j - i);
-	if (m_pView) m_pView->UpdateLabel (m_label);
+	if (m_pView)
+		m_pView->UpdateLabel (m_label);
 }
 
 void gcpDocument::BuildBondList(list<gcpBond*>& BondList, Object* obj)
@@ -167,8 +190,8 @@ bool gcpDocument::ImportOB(OBMol& Mol)
 	OBBond *bond;
 	vector<OBEdgeBase*>::iterator j;
 	bond = Mol.BeginBond(j);
-	double l = floor(DefaultBondLength / bond->GetLength());
-	if (l == 0) l = 1;
+	double l = (bond)? floor(m_BondLength / bond->GetLength()): 1.;
+	if (l == 0.) l = 1.;
 	m_title = g_strdup(Mol.GetTitle());
 	OBAtom *atom;
 	gcpAtom* pAtom;
@@ -177,6 +200,8 @@ bool gcpDocument::ImportOB(OBMol& Mol)
 	vector<OBNodeBase*>::iterator i;
 	for (atom = Mol.BeginAtom(i); atom; atom = Mol.NextAtom(i))
 	{
+		if (atom->GetAtomicNum () == 0)
+			continue;
 		AddAtom(pAtom = new gcpAtom(atom, l));
 		pAtom->GetCoords((double*)&x, (double*)&y);
 		if (valid)
@@ -214,6 +239,8 @@ bool gcpDocument::ImportOB(OBMol& Mol)
 		Id = g_strdup_printf("a%d", bond->GetEndAtomIdx());
 		end = (gcpAtom*)GetDescendant(Id);//Don't verify it is really an atom?
 		g_free(Id);
+		if (!end)
+			continue;
 		order = (unsigned char)(bond->GetBO());
 		if ((pBond = (gcpBond*)begin->GetBond(end)) != NULL)
 		{
@@ -233,7 +260,8 @@ bool gcpDocument::ImportOB(OBMol& Mol)
 			AddBond(pBond);
 		}
 	}
-	m_pApp->ActivateMenu ("Image", HasChildren ());
+	if (m_Window)
+		m_Window->ActivateActionWidget ("/MainMenu/FileMenu/SaveAsImage", HasChildren ());
 	m_pView->Update (this);
 	Update ();
 	m_pView->EnsureSize ();
@@ -264,6 +292,7 @@ void gcpDocument::ExportOB()
 	BuildAtomTable(AtomTable, this, index);
 	Mol.ReserveAtoms(AtomTable.size());
 	Mol.SetTitle((char*)GetTitle());
+	Mol.SetDimension (2);
 	for (i = AtomTable.begin(); i != AtomTable.end(); i++)
 	{
 		pgAtom = (gcpAtom*)GetDescendant((*i).first.data());
@@ -405,13 +434,13 @@ void gcpDocument::AddBond(gcpBond* pBond)
 		while (GetDescendant(id) != NULL);
 		pBond->SetId(id);
 	}
+	AddChild (pBond);
 	gcpAtom *pAtom0 = (gcpAtom*)pBond->GetAtom(0), *pAtom1 = (gcpAtom*)pBond->GetAtom(1);
 	m_pView->Update(pAtom0);
 	m_pView->Update(pAtom1);
 	m_pView->AddObject(pBond);
 	if (m_bIsLoading)
 		return;
-//	AddObject(pBond);
 	//search molecules
 	gcpMolecule * pMol0 = (gcpMolecule*)pAtom0->GetMolecule(),  *pMol1 = (gcpMolecule*)pAtom1->GetMolecule();
 	if (pMol0 && pMol1)
@@ -562,6 +591,8 @@ bool gcpDocument::Load (xmlNodePtr root)
 			xmlFree (tmp);
 		}
 	}
+	if (m_Window)
+		m_Window->SetTitle (GetTitle ());
 	node = GetNodeByName (root, "author");
 	if (node)
 	{
@@ -602,7 +633,8 @@ bool gcpDocument::Load (xmlNodePtr root)
 	m_pView->Update (this);
 	Update ();
 	m_bIsLoading = false;
-	m_pApp->ActivateMenu ("Image", HasChildren ());
+	if (m_Window)
+		m_Window->ActivateActionWidget ("/MainMenu/FileMenu/SaveAsImage", HasChildren ());
 	m_pView->EnsureSize ();
 	return true;
 }
@@ -623,7 +655,7 @@ xmlDocPtr gcpDocument::BuildXMLTree()
 	if (xml == NULL)
 		throw (int) 0;
 	
-	xml->children =  xmlNewDocNode (xml, NULL, (xmlChar*) "chemistry", NULL);
+	xmlDocSetRootElement (xml,  xmlNewDocNode (xml, NULL, (xmlChar*) "chemistry", NULL));
 	ns = xmlNewNs (xml->children, (xmlChar*) "http://www.nongnu.org/gchempaint", (xmlChar*) "gcp");
 	xmlSetNs (xml->children, ns);
 	if (!g_date_valid (&CreationDate))
@@ -692,7 +724,7 @@ void gcpDocument::RemoveAtom(gcpAtom* pAtom)
 {
 	std::map<Atom*, Bond*>::iterator i;
 	gcpBond* pBond;
-	while (pBond = (gcpBond*)pAtom->GetFirstBond(i))
+	while ((pBond = (gcpBond*)pAtom->GetFirstBond(i)))
 	{
 		if (!m_bUndoRedo) m_pCurOp->AddObject(pBond);
 		RemoveBond(pBond);
@@ -708,7 +740,7 @@ void gcpDocument::RemoveFragment(gcpFragment* pFragment)
 	std::map<Atom*, Bond*>::iterator i;
 	gcpAtom* pAtom = pFragment->GetAtom();
 	gcpBond* pBond;
-	while (pBond = (gcpBond*)pAtom->GetFirstBond(i))
+	while ((pBond = (gcpBond*)pAtom->GetFirstBond(i)))
 	{
 		if (!m_bUndoRedo) m_pCurOp->AddObject(pBond);
 		RemoveBond(pBond);
@@ -884,7 +916,7 @@ void gcpDocument::AddObject(Object* pObject)
 	m_pView->AddObject(pObject);
 	if (m_bIsLoading || m_bUndoRedo) return;
 	if (!m_pCurOp) {
-		m_pCurOp = new gcpAddOperation(this);
+		m_pCurOp = new gcpAddOperation(this, ++m_OpID);
 		m_pCurOp->AddObject(pObject);
 	}
 }
@@ -899,11 +931,11 @@ void gcpDocument::OnUndo()
 		Op->Undo();
 		m_UndoList.pop_front();
 		m_RedoList.push_front(Op);
-		m_pApp->ActivateMenu("Redo", true);
+		m_Window->ActivateActionWidget ("/MainMenu/EditMenu/Redo", true);
 	}
 	if (m_UndoList.empty())
-		m_pApp->ActivateMenu("Undo", false);
-	m_pApp->ActivateMenu ("Image", HasChildren ());
+		m_Window->ActivateActionWidget ("/MainMenu/EditMenu/Undo", false);
+	m_Window->ActivateActionWidget ("/MainMenu/FileMenu/SaveAsImage", HasChildren ());
 	m_bUndoRedo = false;
 	Update ();
 	EmptyTranslationTable ();
@@ -919,11 +951,11 @@ void gcpDocument::OnRedo()
 		Op->Redo();
 		m_RedoList.pop_front();
 		m_UndoList.push_front(Op);
-		m_pApp->ActivateMenu("Undo", true);
+		m_Window->ActivateActionWidget ("/MainMenu/EditMenu/Undo", true);
 	}
 	if (m_RedoList.empty())
-		m_pApp->ActivateMenu("Redo", false);
-	m_pApp->ActivateMenu ("Image", HasChildren ());
+		m_Window->ActivateActionWidget ("/MainMenu/EditMenu/Redo", false);
+	m_Window->ActivateActionWidget ("/MainMenu/FileMenu/SaveAsImage", HasChildren ());
 	m_bUndoRedo = false;
 	EmptyTranslationTable ();
 }
@@ -932,16 +964,16 @@ void gcpDocument::FinishOperation()
 {
 	if (!m_pCurOp) return;//FIXME: should at least emit a warning
 	m_UndoList.push_front(m_pCurOp);
-	m_pApp->ActivateMenu("Undo", true);
+	m_Window->ActivateActionWidget ("/MainMenu/EditMenu/Undo", true);
 	while (!m_RedoList.empty())
 	{
 		delete m_RedoList.front();
 		m_RedoList.pop_front();
 	}
-	m_pApp->ActivateMenu("Redo", false);
+	m_Window->ActivateActionWidget ("/MainMenu/EditMenu/Redo", false);
 	m_pCurOp = NULL;
 	m_bDirty = true;
-	m_pApp->ActivateMenu ("Image", HasChildren ());
+	m_Window->ActivateActionWidget ("/MainMenu/FileMenu/SaveAsImage", HasChildren ());
 	Update ();
 }
 
@@ -957,7 +989,8 @@ void gcpDocument::PopOperation()
 	{
 		delete m_UndoList.front();
 		m_UndoList.pop_front();
-		if (m_UndoList.empty()) m_pApp->ActivateMenu("Undo", false);
+		if (m_UndoList.empty())
+			m_Window->ActivateActionWidget ("/MainMenu/EditMenu/Undo", false);
 	}
 }
 
@@ -977,18 +1010,20 @@ void gcpDocument::PushOperation(gcpOperation* operation, bool undo)
 			m_RedoList.pop_front();
 		}
 		m_RedoList.push_front(operation);
-		m_pApp->ActivateMenu("Redo", true);
+		m_Window->ActivateActionWidget ("/MainMenu/EditMenu/Redo", true);
 	}
 	m_pCurOp = NULL;
 }
 
 void gcpDocument::SetActive ()
 {
-	m_pApp->ActivateMenu("Undo", !m_UndoList.empty ());
-	m_pApp->ActivateMenu("Redo", !m_RedoList.empty ());
-	m_pApp->ActivateMenu("Image", HasChildren ());
-	m_pApp->ActivateMenu ("Save", !m_bReadOnly);
-	m_pApp->ActivateToolItem ("Save", !m_bReadOnly);
+	if (m_Window) {
+		m_Window->ActivateActionWidget ("/MainMenu/EditMenu/Undo", !m_UndoList.empty ());
+		m_Window->ActivateActionWidget ("/MainMenu/EditMenu/Redo", !m_RedoList.empty ());
+		m_Window->ActivateActionWidget ("/MainMenu/FileMenu/SaveAsImage", HasChildren ());
+		m_Window->ActivateActionWidget ("/MainMenu/FileMenu/Save", !m_bReadOnly);
+		m_Window->ActivateActionWidget ("/MainToolbar/Save", !m_bReadOnly);
+	}
 }
 
 extern xmlDocPtr pXmlDoc;
@@ -1042,14 +1077,15 @@ void gcpDocument::LoadObjects(xmlNodePtr node)
 
 gcpOperation* gcpDocument::GetNewOperation(gcpOperationType type)
 {
+	m_OpID++;
 	switch (type)
 	{
 		case GCP_ADD_OPERATION:
-			return m_pCurOp = new gcpAddOperation(this);
+			return m_pCurOp = new gcpAddOperation(this, m_OpID);
 		case GCP_DELETE_OPERATION:
-			return m_pCurOp = new gcpDeleteOperation(this);
+			return m_pCurOp = new gcpDeleteOperation(this, m_OpID);
 		case GCP_MODIFY_OPERATION:
-			return m_pCurOp = new gcpModifyOperation(this);
+			return m_pCurOp = new gcpModifyOperation(this, m_OpID);
 		default:
 			return NULL;
 	}
@@ -1083,17 +1119,17 @@ void gcpDocument::AddData(xmlNodePtr node)
 	FinishOperation();
 }
 
-void gcpDocument::ExportImage (const char* filename, const char* type)
+void gcpDocument::ExportImage (string const &filename, const char* type, int resolution)
 {
-	m_pView->ExportImage (filename, type);
+	m_pView->ExportImage (filename, type, resolution);
 }
 
 void gcpDocument::SetReadOnly (bool ro)
 {
 	m_bReadOnly = ro;
 	if (m_pApp->GetActiveDocument () == this) {
-		m_pApp->ActivateMenu ("Save", !ro);
-		m_pApp->ActivateToolItem ("Save", !ro);
+		m_Window->ActivateActionWidget ("/MainMenu/FileMenu/Save", !ro);
+		m_Window->ActivateActionWidget ("/MainToolbar/Save", !ro);
 	}
 }
 
@@ -1110,3 +1146,24 @@ double gcpDocument::GetYAlign ()
 		return (rect.y1 - rect.y0) / 2.;
 	}
 }
+
+void gcpDocument::SetTheme (gcpTheme *theme)
+{
+	m_Theme = theme;
+	m_BondLength = theme->GetBondLength ();
+	m_BondAngle = theme->GetBondAngle ();
+	m_ArrowLength = theme->GetArrowLength ();
+	m_TextFontFamily = theme->GetTextFontFamily ();
+	m_TextFontStyle = theme->GetTextFontStyle ();
+	m_TextFontWeight = theme->GetTextFontWeight ();
+	m_TextFontVariant = theme->GetTextFontVariant ();
+	m_TextFontStretch = theme->GetTextFontStretch ();
+	m_TextFontSize = theme->GetTextFontSize ();
+	pango_attr_list_unref (m_PangoAttrList);
+	m_PangoAttrList = pango_attr_list_new ();
+	pango_attr_list_insert (m_PangoAttrList, pango_attr_family_new (theme->GetFontFamily ()));
+	pango_attr_list_insert (m_PangoAttrList, pango_attr_style_new (theme->GetFontStyle ()));
+	pango_attr_list_insert (m_PangoAttrList, pango_attr_weight_new (theme->GetFontWeight ()));
+	pango_attr_list_insert (m_PangoAttrList, pango_attr_stretch_new (theme->GetFontStretch ()));
+	pango_attr_list_insert (m_PangoAttrList, pango_attr_variant_new (theme->GetFontVariant ()));
+}
