@@ -4,7 +4,7 @@
  * GChemPaint library
  * reactant.cc 
  *
- * Copyright (C) 2002-2007 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2002-2006 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -28,13 +28,14 @@
 #include "application.h"
 #include "document.h"
 #include "text.h"
+#include "theme.h"
 #include <glib/gi18n.h>
 
 extern xmlDocPtr pXmlDoc;
 
 gcpReactant::gcpReactant (): Object (ReactantType)
 {
-	SetId (const_cast<char*> ("r1"));
+	SetId ("r1");
 	m_Stoich = 0;
 	Child = NULL;
 	Stoichiometry = NULL;
@@ -42,7 +43,7 @@ gcpReactant::gcpReactant (): Object (ReactantType)
 
 gcpReactant::gcpReactant (gcpReactionStep* step, Object *object)	throw (invalid_argument): Object (ReactantType)
 {
-	SetId (const_cast<char*> ("r1"));
+	SetId ("r1");
 	step->AddChild (this);
 	GetDocument ()->EmptyTranslationTable();
 	static const set<TypeId>& allowed_types = Object::GetRules ("reactant", RuleMayContain);
@@ -155,30 +156,31 @@ double gcpReactant::GetYAlign ()
 
 void gcpReactant::AddStoichiometry ()
 {
-	gcpDocument *pDoc = (gcpDocument*) GetDocument ();
+	gcpDocument *pDoc = dynamic_cast <gcpDocument*> (GetDocument ());
 	gcpApplication * pApp = pDoc->GetApplication ();
 	gcpView *pView = pDoc->GetView ();
+	gcpTheme *pTheme= pDoc->GetTheme ();
 	gcpWidgetData *pData = (gcpWidgetData*) g_object_get_data (G_OBJECT (pDoc->GetWidget ()), "data");
 	ArtDRect rect;
 	pData->GetObjectBounds (this, &rect);
-	double x = rect.x0 / pData->ZoomFactor;
+	double x = rect.x0 / pTheme->GetZoomFactor ();
 	gcpText *text = new gcpText (x, GetYAlign () + pView->GetBaseLineOffset ());
 	Stoichiometry = text;
 	AddChild (text);
 	pDoc->AddObject (text);
 	pData->GetObjectBounds (text, &rect);
-	Child->Move (rect.x1 / pData->ZoomFactor + pView->GetStoichiometryPadding () - x, 0.);
+	Child->Move (rect.x1 / pTheme->GetZoomFactor () + pTheme->GetStoichiometryPadding () - x, 0.);
 	gcpTool *tool = pApp->GetTool ("Text");
 	GetParent ()->EmitSignal (OnChangedSignal);
 	pApp->ActivateTool ("Text", true);
-	tool->OnClicked (pView, text, rect.x0 * pData->ZoomFactor, GetYAlign () * pData->ZoomFactor, 0);
+	tool->OnClicked (pView, text, rect.x0 * pTheme->GetZoomFactor (), GetYAlign () * pTheme->GetZoomFactor (), 0);
 }
 
 bool gcpReactant::OnSignal (SignalId Signal, Object *Obj)
 {
 	if (Signal == OnChangedSignal) {
 		gcpDocument *pDoc = (gcpDocument*) GetDocument ();
-		gcpView *pView = pDoc->GetView ();
+		gcpTheme *pTheme= pDoc->GetTheme ();
 		gcpWidgetData *pData = (gcpWidgetData*) g_object_get_data (G_OBJECT (pDoc->GetWidget ()), "data");
 		ArtDRect rect;
 		unsigned n = GetChildrenNumber ();
@@ -201,13 +203,11 @@ bool gcpReactant::OnSignal (SignalId Signal, Object *Obj)
 			// Just need to space the two children
 			gnome_canvas_update_now (GNOME_CANVAS (pData->Canvas));
 			pData->GetObjectBounds (Stoichiometry, &rect);
-			double x = rect.x1 / pData->ZoomFactor + pView->GetStoichiometryPadding ();
+			double x = rect.x1 / pTheme->GetZoomFactor () + pTheme->GetStoichiometryPadding ();
 			pData->GetObjectBounds (Child, &rect);
-			Child->Move (x - rect.x0 / pData->ZoomFactor, 0.);
-			GtkTextBuffer *buf = (GtkTextBuffer*) ((gcpTextObject*) Stoichiometry)->GetTextBuffer ();
-			GtkTextIter start, end;
-			gtk_text_buffer_get_bounds (buf, &start, &end);
-			const char *txt = gtk_text_buffer_get_text (buf, &start, &end, true);
+			Child->Move (x - rect.x0 / pTheme->GetZoomFactor (), 0.);
+			PangoLayout *layout = ((gcpTextObject*) Stoichiometry)->GetLayout ();
+			const char *txt = pango_layout_get_text (layout);
 			char *endptr;
 			int n = strtol (txt, &endptr, 10);
 			m_Stoich = (!*endptr)? n: 0;
