@@ -4,7 +4,7 @@
  * GChemPaint library
  * reaction-step.cc 
  *
- * Copyright (C) 2004-2005 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2004-2006 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -29,6 +29,7 @@
 #include "reaction-arrow.h"
 #include "reaction-operator.h"
 #include "document.h"
+#include "theme.h"
 #include "view.h"
 #include "widgetdata.h"
 #include "libgnomecanvas/gnome-canvas.h"
@@ -46,7 +47,8 @@ gcpReactionStep::gcpReactionStep (gcpReaction *reaction, map<double, Object*>& C
 	SetId ("rs1");
 	reaction->AddChild (this);
 	GetDocument ()->EmptyTranslationTable ();
-	gcpDocument *pDoc = (gcpDocument*) GetDocument ();
+	gcpDocument *pDoc = dynamic_cast <gcpDocument*> (GetDocument ());
+	gcpTheme *pTheme = pDoc->GetTheme ();
 	gcpWidgetData  *pData= (gcpWidgetData*) g_object_get_data (G_OBJECT (pDoc->GetWidget ()), "data");
 	map<double, Object*>::iterator im, endm;
 	double x, y, x0, y0, x1, y1;
@@ -59,21 +61,21 @@ gcpReactionStep::gcpReactionStep (gcpReaction *reaction, map<double, Object*>& C
 	x = rect->x1;
 	y = (*im).second->GetYAlign ();
 	for (im++; im != endm; im++) {
-		x += pData->SignPadding;
+		x += pTheme->GetSignPadding ();
 		//Add a sign
 		gcpReactionOperator* pOp = new gcpReactionOperator ();
 		AddChild (pOp);
-		pOp->SetCoords (x / pData->ZoomFactor, y);
+		pOp->SetCoords (x / pTheme->GetZoomFactor (), y);
 		pDoc->AddObject (pOp);
 		gnome_canvas_update_now (GNOME_CANVAS (pData->Canvas));
 		gnome_canvas_item_get_bounds (GNOME_CANVAS_ITEM (pData->Items[pOp]), &x0, &y0, &x1, &y1);
-		pOp->Move ((x - x0) / pData->ZoomFactor, 0);
-		x += pData->SignPadding + x1 - x0;
+		pOp->Move ((x - x0) / pTheme->GetZoomFactor (), 0);
+		x += pTheme->GetSignPadding () + x1 - x0;
 		cur = (*im).second;
 		new gcpReactant (this, cur);
 		rect = &Objects[cur];
 		y0 = cur->GetYAlign ();
-		cur->Move ((x - rect->x0) / pData->ZoomFactor, y - y0);
+		cur->Move ((x - rect->x0) / pTheme->GetZoomFactor (), y - y0);
 		x+= rect->x1 - rect->x0;
 	}
 	Update (pData->Canvas);
@@ -132,8 +134,10 @@ xmlNodePtr gcpReactionStep::Save (xmlDocPtr xml)
 	xmlNodePtr child;
 	while (obj) {
 		if ((*i).second->GetType () != ReactionOperatorType) {
-			if ((child = (*i).second->Save (xml))) xmlAddChild (node, child);
-			else return NULL;
+			if ((child = (*i).second->Save (xml)))
+				xmlAddChild (node, child);
+			else
+				return NULL;
 		}
 		obj = GetNextChild (i);
 	}
@@ -150,7 +154,8 @@ bool gcpReactionStep::Load (xmlNodePtr node)
 	ArtDRect rect;
 	map<string, Object*>::iterator i;
 	Object *pObj = GetFirstChild (i);
-	gcpDocument *pDoc = (gcpDocument*) GetDocument ();
+	gcpDocument *pDoc = dynamic_cast <gcpDocument*> (GetDocument ());
+	gcpTheme *pTheme = pDoc->GetTheme ();
 	gcpWidgetData  *pData= (gcpWidgetData*) g_object_get_data (G_OBJECT (pDoc->GetWidget ()), "data");
 	map<double, Object*>::iterator im, endm;
 	double x, y, x0, y0, x1, y1;
@@ -170,16 +175,16 @@ bool gcpReactionStep::Load (xmlNodePtr node)
 	x = rect.x1;
 	y = (*im).second->GetYAlign ();
 	for (im++; im != endm; im++) {
-		x += pData->SignPadding;
+		x += pTheme->GetSignPadding ();
 		//Add a sign
 		gcpReactionOperator* pOp = new gcpReactionOperator();
 		AddChild (pOp);
-		pOp->SetCoords(x / pData->ZoomFactor, y);
+		pOp->SetCoords(x / pTheme->GetZoomFactor (), y);
 		pDoc->AddObject(pOp);
 		gnome_canvas_update_now (GNOME_CANVAS (pData->Canvas));
 		gnome_canvas_item_get_bounds (GNOME_CANVAS_ITEM (pData->Items[pOp]), &x0, &y0, &x1, &y1);
-		pOp->Move ((x - x0) / pData->ZoomFactor, 0);
-		x += pData->SignPadding + x1 - x0;
+		pOp->Move ((x - x0) / pTheme->GetZoomFactor (), 0);
+		x += pTheme->GetSignPadding () + x1 - x0;
 		pObj = (*im).second;
 		rect = Objects[pObj];
 		x+= rect.x1 - rect.x0;
@@ -207,7 +212,8 @@ bool gcpReactionStep::OnSignal (SignalId Signal, Object *Child)
 		ArtDRect rect;
 		map<string, Object*>::iterator i;
 		Object *pObj = GetFirstChild (i);
-		gcpDocument *pDoc = (gcpDocument*) GetDocument ();
+		gcpDocument *pDoc = dynamic_cast <gcpDocument*> (GetDocument ());
+		gcpTheme *pTheme = pDoc->GetTheme ();
 		gcpView *pView = pDoc->GetView ();
 		gcpWidgetData  *pData= (gcpWidgetData*) g_object_get_data (G_OBJECT (pDoc->GetWidget ()), "data");
 		map<double, Object*>::iterator im, endm;
@@ -238,19 +244,19 @@ bool gcpReactionStep::OnSignal (SignalId Signal, Object *Child)
 		x = rect.x1;
 		y = (*im).second->GetYAlign ();
 		for (im++; im != endm; im++) {
-			x += pData->SignPadding;
+			x += pTheme->GetSignPadding ();
 			//Add a sign
 			gcpReactionOperator* pOp = new gcpReactionOperator();
 			AddChild (pOp);
-			pOp->SetCoords(x / pData->ZoomFactor, y);
+			pOp->SetCoords(x / pTheme->GetZoomFactor (), y);
 			pDoc->AddObject(pOp);
 			gnome_canvas_update_now (GNOME_CANVAS (pData->Canvas));
 			gnome_canvas_item_get_bounds (GNOME_CANVAS_ITEM (pData->Items[pOp]), &x0, &y0, &x1, &y1);
-			pOp->Move ((x - x0) / pData->ZoomFactor, 0);
-			x += pData->SignPadding + x1 - x0;
+			pOp->Move ((x - x0) / pTheme->GetZoomFactor (), 0);
+			x += pTheme->GetSignPadding () + x1 - x0;
 			pObj = (*im).second;
 			rect = Objects[pObj];
-			pObj->Move ((x - rect.x0) / pData->ZoomFactor, y - pObj->GetYAlign ());
+			pObj->Move ((x - rect.x0) / pTheme->GetZoomFactor (), y - pObj->GetYAlign ());
 			x+= rect.x1 - rect.x0;
 		}
 		Update (pData->Canvas);
