@@ -4,7 +4,7 @@
  * GChemPaint libray
  * settings.h 
  *
- * Copyright (C) 2001-2004 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2001-2006 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -28,7 +28,9 @@
 #include <glib.h>
 #include <pango/pango.h>
 
+/*
 extern double DefaultBondLength;
+extern double DefaultBondAngle;
 extern double DefaultBondDist;
 extern double DefaultBondWidth;
 extern double DefaultArrowLength;
@@ -45,16 +47,17 @@ extern double DefaultArrowPadding;
 extern double DefaultStoichiometryPadding;
 extern double DefaultObjectPadding;
 extern double DefaultSignPadding;
-extern double ChargeSignHeight;
-extern double ChargeSignSize;
-extern double ChargeYAlign;
+extern double DefaultChargeSignHeight;
+extern double DefaultChargeSignSize;
+extern double DefaultChargeYAlign;*/
 extern gchar *Color, *DeleteColor, *AddColor, *SelectColor;
-extern gchar* DefaultFontFamily;
+/*extern gchar *DefaultFontFamily, *DefaultTextFontFamily;
 extern PangoStyle DefaultFontStyle;
 extern PangoWeight DefaultFontWeight;
 extern PangoVariant DefaultFontVariant;
 extern PangoStretch DefaultFontStretch;
-extern gint DefaultFontSize;
+extern gint DefaultFontSize;*/
 extern unsigned MaxStackSize;
+extern bool MergeAtoms;
 
 #endif // GCHEMPAINT_SETTINGS_H
