@@ -43,15 +43,13 @@ public:
 	void SetPage (gcpTool *tool, int i);
 	void OnSelectTool (gcpTool *tool);
 	void RegisterTool (GtkWidget *w);
-	void Focus (bool focus);
-	bool GetFocused () {return focused;}
+	void OnElementChanged (int Z);
 
 private:
 	GtkUIManager *m_UIManager;
 	GtkBox *m_ButtonsBox;
 	map<gcpTool*, int> m_Pages;
 	GtkNotebook *m_Book;
-	bool focused;
 };
 
 #endif // GCHEMPAINT_TOOLS_H
