@@ -25,6 +25,7 @@
 #include "document.h"
 #include "window.h"
 #include "about.h"
+#include "preferences.h"
 #include "zoomdlg.h"
 #include <gcu/filechooser.h>
 #include <libgnomeprint/gnome-print.h>
@@ -34,6 +35,7 @@
 
 static void on_destroy (GtkWidget* widget, gcpWindow* Win)
 {
+	Win->GetDocument ()->GetView ()->PrepareUnselect ();
 	delete Win;
 }
 
@@ -117,6 +119,11 @@ static void on_delete_selection(GtkWidget* widget, gcpWindow* Win)
 	Win->OnDeleteSelection();
 }
 
+static void on_preferences (GtkWidget* widget, gcpWindow* Win)
+{
+	Win->OnPreferences ();
+}
+
 /*static bool on_key_release(GtkWidget* widget, GdkEventKey* ev, gcpWindow* Win)
 {
 	return Win->OnKeyReleased(widget, ev);
@@ -181,6 +188,7 @@ static void on_zoom (GtkWidget* widget, gcpWindow* Win)
 
 static bool on_focus_in (GtkWidget *widget, GdkEventFocus *event, gcpWindow* Win)
 {
+	Win->GetDocument ()->GetView ()->ShowCursor (true);
 	gcpApplication *App = Win->GetApplication ();
 	App->NotifyFocus (true, Win);
 	gtk_clipboard_request_contents (gtk_clipboard_get (GDK_SELECTION_CLIPBOARD), gdk_atom_intern ("TARGETS", FALSE),  (GtkClipboardReceivedFunc) on_receive_targets, App);
@@ -189,6 +197,7 @@ static bool on_focus_in (GtkWidget *widget, GdkEventFocus *event, gcpWindow* Win
 
 static bool on_focus_out (GtkWidget *widget, GdkEventFocus *event, gcpWindow* Win)
 {
+	Win->GetDocument ()->GetView ()->ShowCursor (false);
 	Win->GetApplication ()->NotifyFocus (false);
 	return false;
 }
@@ -269,6 +278,15 @@ static bool on_key_press(GtkWidget* widget, GdkEventKey* ev, gcpWindow* Win)
 	return Win->OnKeyPressed(widget, ev);
 }
 
+static void on_recent (GtkRecentChooser *widget, gcpWindow *Win)
+{
+	gcpApplication *App = Win->GetApplication ();
+	GtkRecentInfo *info = gtk_recent_chooser_get_current_item (widget);
+	gcpDocument *pDoc = Win->GetDocument ();
+	App->FileProcess (gtk_recent_info_get_uri (info), gtk_recent_info_get_mime_type (info), false, NULL, (!pDoc->HasChildren () && !pDoc->IsDirty ())? pDoc: NULL);
+	gtk_recent_info_unref (info);
+}
+
 /*********
  * Menus *
  *********/
@@ -303,10 +321,12 @@ static GtkActionEntry entries[] = {
 		  N_("Copy the selection"), G_CALLBACK (on_copy_selection) },
 	  { "Paste", GTK_STOCK_PASTE, N_("_Paste"), "<control>V",
 		  N_("Paste the clipboard"), G_CALLBACK (on_paste_selection) },
-	  { "Delete", GTK_STOCK_CLEAR, N_("C_lear"), NULL,
+	  { "Erase", GTK_STOCK_CLEAR, N_("C_lear"), NULL,
 		  N_("Clear the selection"), G_CALLBACK (on_delete_selection) },
 	  { "SelectAll", NULL, N_("Select _All"), "<control>A",
 		  N_("Select everything"), G_CALLBACK (on_select_all) },
+	  { "Preferences", GTK_STOCK_PREFERENCES, N_("Pr_eferences..."), NULL,
+		  N_("Configure the application"), G_CALLBACK (on_preferences) },
   { "ViewMenu", NULL, N_("_View") },
 	  { "ZoomMenu", NULL, N_("_Zoom") },
 		  { "400%", NULL, N_("_400%"), "<control>4",
@@ -369,9 +389,11 @@ static const char *ui_description =
 "      <menuitem action='Cut'/>"
 "      <menuitem action='Copy'/>"
 "      <menuitem action='Paste'/>"
-"      <menuitem action='Delete'/>"
+"      <menuitem action='Erase'/>"
 "      <separator name='edit-sep2'/>"
 "      <menuitem action='SelectAll'/>"
+"      <separator name='edit-sep3'/>"
+"      <menuitem action='Preferences'/>"
 "    </menu>"
 "    <menu action='ViewMenu'>"
 "      <menu action='ZoomMenu'>"
@@ -442,13 +464,9 @@ gcpWindow::gcpWindow (gcpApplication *App, char const *extra_ui)
 	GError *error;
 
 	m_App = App;
-	m_Doc = new gcpDocument (App, true, this);
-	GtkWidget *w = m_Doc->GetView ()->CreateNewWidget ();
 	m_Window = GTK_WINDOW (gtk_window_new (GTK_WINDOW_TOPLEVEL));
+	g_object_set (G_OBJECT (m_Window), "urgency-hint", false, NULL);
 	g_object_set_data (G_OBJECT (m_Window), "gcp-role", (void*) 1);
-	gtk_window_set_position (m_Window, GTK_WIN_POS_MOUSE);
-	gtk_window_set_title (m_Window, m_Doc->GetTitle ());
-	gtk_window_set_icon_name (m_Window, "gchempaint");
 	g_signal_connect (G_OBJECT (m_Window), "destroy", G_CALLBACK (on_destroy), this);
 	g_signal_connect (G_OBJECT (m_Window), "focus_in_event", G_CALLBACK (on_focus_in), this);
 	g_signal_connect (G_OBJECT (m_Window), "focus_out_event", G_CALLBACK (on_focus_out), this);
@@ -489,18 +507,36 @@ gcpWindow::gcpWindow (gcpApplication *App, char const *extra_ui)
 		g_error_free (error);
 	}
 
+	GtkWidget *menu = gtk_ui_manager_get_widget (m_UIManager, "/MainMenu/FileMenu/Open");
+	GtkWidget *w = gtk_recent_chooser_menu_new_for_manager (App->GetRecentManager ());
+	GtkRecentFilter *filter = gtk_recent_filter_new ();
+	gtk_recent_filter_add_mime_type (filter, "application/x-gchempaint");
+	gtk_recent_filter_add_mime_type (filter, "chemical/x-chemdraw");
+	gtk_recent_filter_add_mime_type (filter, "chemical/x-cml");
+	gtk_recent_filter_add_mime_type (filter, "chemical/x-mdl-molfile");
+	gtk_recent_filter_add_mime_type (filter, "chemical/x-pdb");
+	gtk_recent_chooser_add_filter (GTK_RECENT_CHOOSER (w), filter);
+	g_signal_connect (G_OBJECT (w), "item-activated", G_CALLBACK (on_recent), this);
+	GtkWidget *item = gtk_menu_item_new_with_label (_("Open recent"));
+	gtk_menu_item_set_submenu (GTK_MENU_ITEM (item), w);
+	gtk_widget_show_all (item);
+	gtk_menu_shell_insert (GTK_MENU_SHELL (gtk_widget_get_parent (menu)), item, 3);
+
 	bar = gtk_ui_manager_get_widget (m_UIManager, "/MainMenu");
 	gtk_box_pack_start (GTK_BOX (vbox), bar, false, false, 0);
 	bar = gtk_ui_manager_get_widget (m_UIManager, "/MainToolbar");
 	gtk_toolbar_set_tooltips(GTK_TOOLBAR(bar), true);
-	gtk_box_pack_start (GTK_BOX (vbox), bar, true, true, 0);
+	gtk_box_pack_start (GTK_BOX (vbox), bar, false, false, 0);
+	m_Doc = new gcpDocument (App, true, this);
+	gtk_window_set_title (m_Window, m_Doc->GetTitle ());
+	w = m_Doc->GetView ()->CreateNewWidget ();
 	GtkScrolledWindow* scroll = (GtkScrolledWindow*)gtk_scrolled_window_new(NULL, NULL);
 	gtk_scrolled_window_set_policy(scroll, GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
 	gtk_scrolled_window_set_shadow_type (scroll, GTK_SHADOW_IN);
 	gtk_scrolled_window_add_with_viewport(scroll, w);
 	gtk_widget_set_size_request(GTK_WIDGET(scroll), 408, 308);
 	gtk_widget_show(GTK_WIDGET(scroll));
-	gtk_box_pack_start (GTK_BOX (vbox), GTK_WIDGET (scroll), false, false, 0);
+	gtk_box_pack_start (GTK_BOX (vbox), GTK_WIDGET (scroll), true, true, 0);
 	m_Bar = gtk_statusbar_new ();
 	m_statusId = gtk_statusbar_get_context_id (GTK_STATUSBAR (m_Bar), "status");
 	gtk_statusbar_push (GTK_STATUSBAR (m_Bar), m_statusId, _("Ready"));
@@ -510,6 +546,9 @@ gcpWindow::gcpWindow (gcpApplication *App, char const *extra_ui)
 	g_signal_connect(GTK_OBJECT(m_Window), "key_press_event", (GCallback)on_key_press, this);
 	g_signal_connect(GTK_OBJECT(m_Window), "key_release_event", (GCallback)on_key_release, this);
 
+	gtk_widget_set_sensitive (gtk_ui_manager_get_widget (m_UIManager, "/MainMenu/EditMenu/Copy"), false);
+	gtk_widget_set_sensitive (gtk_ui_manager_get_widget (m_UIManager, "/MainMenu/EditMenu/Cut"), false);
+	gtk_widget_set_sensitive (gtk_ui_manager_get_widget (m_UIManager, "/MainMenu/EditMenu/Erase"), false);
 	gtk_widget_show_all (GTK_WIDGET (m_Window));
 	App->SetActiveDocument (m_Doc);
 }
@@ -638,6 +677,11 @@ void gcpWindow::OnDeleteSelection()
 		m_Doc->GetView ()->OnDeleteSelection (m_Doc->GetWidget ());
 }
 
+void gcpWindow::OnPreferences ()
+{
+	new gcpPrefsDlg (GetApplication ());
+}
+
 void gcpWindow::Zoom (double zoom)
 {
 	gcpView *pView = m_Doc->GetView ();
@@ -708,3 +752,10 @@ char const *gcpWindow::GetDefaultTitle ()
 {
 	return _("GChemPaint");
 }
+
+void gcpWindow::ActivateActionWidget (char const *path, bool activate)
+{
+	GtkWidget *w = gtk_ui_manager_get_widget (m_UIManager, path);
+	if (w)
+		gtk_widget_set_sensitive (w, activate);
+}
