@@ -4,9 +4,7 @@
  * GChemPaint cycles plugin
  * cycletool.cc 
  *
- * Copyright (C) 2001-2005
- *
- * Developed by Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2001-2006 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -27,9 +25,9 @@
 #include "gchempaint-config.h"
 #include "cycletool.h"
 #include "lib/settings.h"
-#include "lib/mendeleiev.h"
 #include "lib/document.h"
 #include "lib/application.h"
+#include "lib/theme.h"
 #include <math.h>
 #include <list>
 #include <vector>
@@ -70,7 +68,9 @@ bool gcpCycleTool::OnClicked()
 	double a0, a1, a2, b1, b2, m1, m2;
 	gcpAtom* pAtom, *pAtom1;
 	gcpBond* pBond, *pBond1;
-	m_dLength = DefaultBondLength * m_dZoomFactor;
+	gcpDocument *pDoc = m_pView->GetDoc ();
+	gcpTheme *pTheme = pDoc->GetTheme ();
+	m_dLength = pDoc->GetBondLength () * m_dZoomFactor;
 	map<Atom*, Bond*>::iterator j;
 	int i;
 	bool bDone = false;
@@ -222,7 +222,7 @@ bool gcpCycleTool::OnClicked()
 						pBond = (gcpBond*)pAtom->GetFirstBond(j);
 						orientations.push_back(pBond->GetAngle2DRad(pAtom));
 						unsigned k;
-						while (pBond = (gcpBond*)pAtom->GetNextBond(j))
+						while ((pBond = (gcpBond*)pAtom->GetNextBond(j)))
 						{
 							k = 0;
 							a0 = pBond->GetAngle2DRad(pAtom);
@@ -278,7 +278,7 @@ bool gcpCycleTool::OnClicked()
 								gnome_canvas_polygon_get_type(),
 								"points", points,
 								"outline_color", (m_bAllowed)? AddColor: DeleteColor,
-								"width_units", m_pData->BondWidth,
+								"width_units", pTheme->GetBondWidth (),
 								NULL);
 	return true;
 }
@@ -290,6 +290,8 @@ void gcpCycleTool::OnDrag()
 	double x1, y1, x2, y2;
 	bool bDone = false;
 	GnomeCanvasItem* pItem = gnome_canvas_get_item_at(GNOME_CANVAS(m_pWidget), m_x, m_y);
+	gcpDocument *pDoc = m_pView->GetDoc ();
+	gcpTheme *pTheme = pDoc->GetTheme ();
 	if (pItem == (GnomeCanvasItem*)m_pBackground) pItem = NULL;
 	Object* pObject = NULL;
 	if (pItem) pObject = (Object*)g_object_get_data(G_OBJECT(pItem), "object");
@@ -297,7 +299,7 @@ void gcpCycleTool::OnDrag()
 	{
 		if (m_pObject->GetType() == BondType)
 		{
-			if (((gcpBond*)m_pObject)->GetDist(m_x / m_dZoomFactor, m_y / m_dZoomFactor) < (DefaultPadding + DefaultBondWidth / 2) * m_dZoomFactor)
+			if (((gcpBond*)m_pObject)->GetDist(m_x / m_dZoomFactor, m_y / m_dZoomFactor) < (pTheme->GetPadding () + pTheme->GetBondWidth () / 2) * m_dZoomFactor)
 			{
 				if (m_pItem)
 				{
@@ -361,7 +363,7 @@ void gcpCycleTool::OnDrag()
 					m_End->GetCoords(&x2, &y2);
 					x2 *= m_dZoomFactor;
 					y2 *= m_dZoomFactor;
-					double d = sqrt(square(x2 - x1) + square(y2 - y1)), L = DefaultBondLength * m_dZoomFactor, t;
+					double d = sqrt(square(x2 - x1) + square(y2 - y1)), L = pDoc->GetBondLength () * m_dZoomFactor, t;
 					unsigned n = m_size - m_Chain->GetLength();
 					if (d < L * n)
 					{//FIXME: this algorithm is far from optimal!!!
@@ -423,8 +425,8 @@ void gcpCycleTool::OnDrag()
 				y1 = m_xn[3];
 				for (i = 4; i < m_size * 2; i+= 2)
 				{
-					m_xn[i] = x1 += (DefaultBondLength * m_dZoomFactor) * cos(m_dAngle - m_dDev * (i / 2 - 1));
-					m_xn[i + 1] = y1 -= (DefaultBondLength * m_dZoomFactor) * sin(m_dAngle - m_dDev * (i / 2 - 1));
+					m_xn[i] = x1 += (pDoc->GetBondLength () * m_dZoomFactor) * cos (m_dAngle - m_dDev * (i / 2 - 1));
+					m_xn[i + 1] = y1 -= (pDoc->GetBondLength () * m_dZoomFactor) * sin (m_dAngle - m_dDev * (i / 2 - 1));
 					points->coords[i] = x1;
 					points->coords[i + 1] = y1;
 				}
@@ -501,7 +503,7 @@ void gcpCycleTool::OnDrag()
 			m_dAngle = dAngle * M_PI / 180;
 			x1 = m_x0;
 			y1 = m_y0;
-			double d = (m_nState & GDK_SHIFT_MASK)? sqrt(square(m_x) + square(m_y)): DefaultBondLength * m_dZoomFactor;
+			double d = (m_nState & GDK_SHIFT_MASK)? sqrt (square (m_x) + square (m_y)): pDoc->GetBondLength () * m_dZoomFactor;
 			for (i = 2; i < m_size * 2; i+= 2)
 			{
 				m_xn[i] = x1 += d * cos(m_dAngle - m_dDev * (i / 2 - 1));
@@ -521,7 +523,7 @@ void gcpCycleTool::OnDrag()
 								gnome_canvas_polygon_get_type(),
 								"points", points,
 								"outline_color",  (m_bAllowed)? AddColor: DeleteColor,
-								"width_units", m_pData->BondWidth,
+								"width_units", pTheme->GetBondWidth (),
 								NULL);
 }
 
@@ -550,6 +552,7 @@ void gcpCycleTool::OnRelease()
 	gcpDocument *pDoc = m_pView->GetDoc ();
 	gcpOperation *pOp = NULL;
 	gcpMolecule *pMol = NULL;
+	char const *MolId;
 	int i;
 	for (i = 0; i < m_size; i++)
 	{
@@ -559,7 +562,7 @@ void gcpCycleTool::OnRelease()
 		pItem = gnome_canvas_get_item_at(GNOME_CANVAS(m_pWidget), m_x, m_y);
 		if (pItem == (GnomeCanvasItem*)m_pBackground) pItem = NULL;
 		m_pObject = (pItem)? (Object*)g_object_get_data(G_OBJECT(pItem), "object"): NULL;
-		if (m_pObject)
+		if (MergeAtoms && m_pObject)
 		{
 			if (m_pObject->GetType() == BondType)
 			{
@@ -573,6 +576,9 @@ void gcpCycleTool::OnRelease()
 				if (pMol == NULL) {
 					pMol = reinterpret_cast<gcpMolecule *> (pAtom[i]->GetMolecule ());
 					pMol->Lock (true);
+					// we must store the id, the molecule might be destroyed
+					// FIXME! what happens to the parent in such a case?
+					MolId = pMol->GetId ();
 				}
 				// if group not in ModifiedObjects, add it and save it
 				pObject = pAtom[i]->GetGroup ();
@@ -620,6 +626,8 @@ void gcpCycleTool::OnRelease()
 		pOp->AddObject (pObject);
 	}
 	pDoc->FinishOperation ();
+	if (pMol)
+		pMol = static_cast <gcpMolecule*> (pDoc->GetDescendant (MolId));
 	if (pMol) {
 		pMol->Lock (false);
 		pMol->EmitSignal (OnChangedSignal);
@@ -674,8 +682,9 @@ void gcpCycleTool::OnChangeState()
 			points->coords[3] = m_xn[3] = y1 = y2 * m_dZoomFactor;
 			for (int i = 4; i < m_size * 2; i+= 2)
 			{
-				m_xn[i] = x1 += (DefaultBondLength * m_dZoomFactor) * cos(m_dAngle - m_dDev * (i / 2 - 1));
-				m_xn[i + 1] = y1 -= (DefaultBondLength * m_dZoomFactor) * sin(m_dAngle - m_dDev * (i / 2 - 1));
+				gcpDocument *pDoc = m_pView->GetDoc ();
+				m_xn[i] = x1 += (pDoc->GetBondLength () * m_dZoomFactor) * cos(m_dAngle - m_dDev * (i / 2 - 1));
+				m_xn[i + 1] = y1 -= (pDoc->GetBondLength () * m_dZoomFactor) * sin(m_dAngle - m_dDev * (i / 2 - 1));
 				points->coords[i] = x1;
 				points->coords[i + 1] = y1;
 			}
@@ -711,7 +720,7 @@ bool gcpCycleTool::CheckIfAllowed()
 		pItem = gnome_canvas_get_item_at(GNOME_CANVAS(m_pWidget), m_x, m_y);
 		if (pItem == (GnomeCanvasItem*)m_pBackground) pItem = NULL;
 		pObject = (pItem)? (Object*)g_object_get_data(G_OBJECT(pItem), "object"): NULL;
-		if (pObject)
+		if (MergeAtoms && pObject)
 		{
 			TypeId Id = pObject->GetType();
 			switch (Id)
@@ -740,6 +749,38 @@ bool gcpCycleTool::CheckIfAllowed()
 	return true;
 }
 
+static void on_length_changed (GtkSpinButton *btn, gcpCycleTool *tool)
+{
+	tool->SetLength (gtk_spin_button_get_value (btn));
+}
+
+static void on_merge_toggled (GtkToggleButton *btn)
+{
+	MergeAtoms = gtk_toggle_button_get_active (btn);
+}
+
+void gcpCycleTool::SetLength (double length)
+{
+	m_pApp->GetActiveDocument ()->SetBondLength (length);
+}
+
+GtkWidget *gcpCycleTool::GetPropertyPage ()
+{
+	GladeXML *xml = glade_xml_new (DATADIR"/gchempaint-unstable/ui/cycle.glade", "cycle", GETTEXT_PACKAGE);
+	m_LengthBtn = GTK_SPIN_BUTTON (glade_xml_get_widget (xml, "bond-length"));
+	g_signal_connect (m_LengthBtn, "value-changed", G_CALLBACK (on_length_changed), this);
+	m_MergeBtn = GTK_TOGGLE_BUTTON (glade_xml_get_widget (xml, "merge"));
+	g_signal_connect (m_MergeBtn, "toggled", G_CALLBACK (on_merge_toggled), NULL);
+	return glade_xml_get_widget (xml, "cycle");
+}
+
+void gcpCycleTool::Activate ()
+{
+	gcpDocument *pDoc = m_pApp->GetActiveDocument ();
+	gtk_spin_button_set_value (m_LengthBtn, pDoc->GetBondLength ());
+	gtk_toggle_button_set_active (m_MergeBtn, MergeAtoms);
+}
+
 class gcpCNToolDlg: public Dialog
 {
 public:
@@ -771,6 +812,23 @@ void gcpNCycleTool::SetSize(unsigned char size)
 	if ((m_size = size)) Init();
 }
 
+static void on_size_changed (GtkSpinButton *button, gcpNCycleTool *tool)
+{
+	tool->SetSize ((unsigned char) gtk_spin_button_get_value_as_int (button));
+}
+
+GtkWidget *gcpNCycleTool::GetPropertyPage ()
+{
+	GladeXML *xml = glade_xml_new (DATADIR"/gchempaint-unstable/ui/cyclen.glade", "cycle", GETTEXT_PACKAGE);
+	m_LengthBtn = GTK_SPIN_BUTTON (glade_xml_get_widget (xml, "bond-length"));
+	g_signal_connect (m_LengthBtn, "value-changed", G_CALLBACK (on_length_changed), NULL);
+	m_MergeBtn = GTK_TOGGLE_BUTTON (glade_xml_get_widget (xml, "merge"));
+	g_signal_connect (m_MergeBtn, "toggled", G_CALLBACK (on_merge_toggled), NULL);
+	m_SizeBtn = GTK_SPIN_BUTTON (glade_xml_get_widget (xml, "bond-length"));
+	g_signal_connect (m_SizeBtn, "value-changed", G_CALLBACK (on_size_changed), this);
+	return glade_xml_get_widget (xml, "cycle");
+}
+
 void gcpNCycleTool::SetOptions ()
 {
 	gcpCNToolDlg* dlg = (gcpCNToolDlg*) m_pApp->GetDialog ("CycleN");
