@@ -31,6 +31,7 @@
 #include "lib/document.h"
 #include "lib/application.h"
 #include "lib/theme.h"
+#include "lib/window.h"
 #include <math.h>
 #include <stdexcept>
 
@@ -60,9 +61,9 @@ static void on_merge(GtkWidget *btn, gcpApplication* App)
 
 gcpSelectionTool::gcpSelectionTool(gcpApplication *App): gcpTool(App, "Select")
 {
-	m_pApp->SetCallback("flip", (GCallback) on_flip);
-	m_pApp->SetCallback("rotate", (GCallback) on_rotate);
-	m_pApp->SetCallback("merge", (GCallback) on_merge);
+//	m_pApp->SetCallback("flip", (GCallback) on_flip);
+//	m_pApp->SetCallback("rotate", (GCallback) on_rotate);
+//	m_pApp->SetCallback("merge", (GCallback) on_merge);
 	m_bRotate = false;
 	m_UIManager = NULL;
 }
@@ -75,6 +76,7 @@ gcpSelectionTool::~gcpSelectionTool()
 
 bool gcpSelectionTool::OnClicked()
 {
+	gcpWindow *win = m_pView->GetDoc ()->GetWindow ();
 	if (m_pObject)
 	{
 /*		switch (m_pObject->GetType())
@@ -92,17 +94,17 @@ bool gcpSelectionTool::OnClicked()
 		{
 			m_pData->UnselectAll();
 			m_pData->SetSelected(m_pObject);
-			m_pApp->ActivateMenu("Copy", true);
-			m_pApp->ActivateMenu("Cut", true);
-			m_pApp->ActivateMenu("Erase", true);
+			win->ActivateActionWidget ("/MainMenu/EditMenu/Copy", true);
+			win->ActivateActionWidget ("/MainMenu/EditMenu/Cut", true);
+			win->ActivateActionWidget ("/MainMenu/EditMenu/Erase", true);
 		}
 	}
 	else
 	{
 		m_pData->UnselectAll();
-		m_pApp->ActivateMenu("Copy", false);
-		m_pApp->ActivateMenu("Cut", false);
-		m_pApp->ActivateMenu("Erase", false);
+		win->ActivateActionWidget ("/MainMenu/EditMenu/Copy", false);
+		win->ActivateActionWidget ("/MainMenu/EditMenu/Cut", false);
+		win->ActivateActionWidget ("/MainMenu/EditMenu/Erase", false);
 	}
 	if (m_bRotate) {
 		// Calculate center of selection
@@ -249,10 +251,10 @@ void gcpSelectionTool::Activate()
 	if (m_UIManager)
 		gtk_widget_set_sensitive (m_MergeBtn, false);
 	else {
-		m_pApp->ActivateToolItem ("HorizFlip", true);
-		m_pApp->ActivateToolItem ("VertFlip", true);
-		m_pApp->ActivateToolItem ("Rotate", true);
-		m_pApp->ActivateToolItem ("Merge", false);
+//		m_pApp->ActivateToolItem ("HorizFlip", true);
+//		m_pApp->ActivateToolItem ("VertFlip", true);
+//		m_pApp->ActivateToolItem ("Rotate", true);
+//		m_pApp->ActivateToolItem ("Merge", false);
 	}
 	gcpDocument *pDoc = m_pApp->GetActiveDocument ();
 	if (pDoc) {
@@ -269,10 +271,10 @@ bool gcpSelectionTool::Deactivate()
 		SelectedWidgets.front()->UnselectAll();
 		SelectedWidgets.pop_front();
 	}
-	m_pApp->ActivateToolItem ("HorizFlip", false);
-	m_pApp->ActivateToolItem ("VertFlip", false);
-	m_pApp->ActivateToolItem ("Rotate", false);
-	m_pApp->ActivateToolItem ("Merge", false);
+//	m_pApp->ActivateToolItem ("HorizFlip", false);
+//	m_pApp->ActivateToolItem ("VertFlip", false);
+//	m_pApp->ActivateToolItem ("Rotate", false);
+//	m_pApp->ActivateToolItem ("Merge", false);
 	return true;
 }
 
@@ -281,16 +283,14 @@ void gcpSelectionTool::AddSelection(gcpWidgetData* data)
 	gcpWidgetData *d = m_pData;
 	m_pData = data;
 	m_pView = data->View;
+	gcpWindow *win = m_pView->GetDoc ()->GetWindow ();
 	if (m_pData->HasSelection())
 	{
 		GtkClipboard *clipboard = gtk_clipboard_get (GDK_SELECTION_PRIMARY);
 		m_pView->OnCopySelection (m_pData->Canvas, clipboard);
-		if (!m_pView->IsEmbedded())
-		{
-			m_pApp->ActivateMenu("Copy", true);
-			m_pApp->ActivateMenu("Cut", true);
-			m_pApp->ActivateMenu("Erase", true);
-		}
+		win->ActivateActionWidget ("/MainMenu/EditMenu/Copy", true);
+		win->ActivateActionWidget ("/MainMenu/EditMenu/Cut", true);
+		win->ActivateActionWidget ("/MainMenu/EditMenu/Erase", true);
 	}
 	SelectedWidgets.remove(m_pData);
 	SelectedWidgets.push_front(m_pData);
@@ -303,10 +303,10 @@ void gcpSelectionTool::AddSelection(gcpWidgetData* data)
 		gtk_widget_set_sensitive (m_MergeBtn, ((m_pData->SelectedObjects.size () == 2) &&
 			(m_pData->SelectedObjects.front()->GetType() == MoleculeType) &&
 			(m_pData->SelectedObjects.back()->GetType() == MoleculeType)));
-	else
-		m_pApp->ActivateToolItem ("Merge", ((m_pData->SelectedObjects.size () == 2) &&
-			(m_pData->SelectedObjects.front()->GetType() == MoleculeType) &&
-			(m_pData->SelectedObjects.back()->GetType() == MoleculeType)));
+//	else
+//		m_pApp->ActivateToolItem ("Merge", ((m_pData->SelectedObjects.size () == 2) &&
+//			(m_pData->SelectedObjects.front()->GetType() == MoleculeType) &&
+//			(m_pData->SelectedObjects.back()->GetType() == MoleculeType)));
 }
 
 void gcpSelectionTool::OnFlip (bool horizontal)
@@ -366,7 +366,7 @@ void gcpSelectionTool::Merge ()
 	} else {
 		pDoc->AbortOperation ();
 	}
-	m_pApp->ActivateToolItem ("Merge", false);
+//	m_pApp->ActivateToolItem ("Merge", false);
 }
 
 static void on_create_group (gcpSelectionTool* tool)
